@@ -4,7 +4,7 @@
  *
  * This file is part of the GNOME Devtools Libraries.
  *
- * Copyright (C) 2002 Gustavo Girldez <gustavo.giraldez@gmx.net>
+ * Copyright (C) 2002 Gustavo Giráldez <gustavo.giraldez@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -94,7 +94,8 @@ enum {
     PROP_STOCK_ID,
     PROP_PIXBUF_ICON,
     PROP_MASTER,
-    PROP_EXPORT_PROPERTIES
+    PROP_EXPORT_PROPERTIES,
+    PROP_LAST
 };
 
 enum {
@@ -105,6 +106,8 @@ enum {
 
 static guint gdl_dock_object_signals [LAST_SIGNAL] = { 0 };
 
+static GParamSpec *properties[PROP_LAST];
+
 struct _GdlDockObjectPrivate {
     guint               automatic : 1;
     guint               attached : 1;
@@ -198,37 +201,43 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
      * The object name.  If the object is manual the name can be used
      * to recall the object from any other object in the ring
      */
-    g_object_class_install_property (
-        object_class, PROP_NAME,
+    properties[PROP_NAME] =
         g_param_spec_string (GDL_DOCK_NAME_PROPERTY, _("Name"),
                              _("Unique name for identifying the dock object"),
                              NULL,
                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                             GDL_DOCK_PARAM_EXPORT));
+                             GDL_DOCK_PARAM_EXPORT);
+
+    g_object_class_install_property (
+        object_class, PROP_NAME, properties[PROP_NAME]);
 
     /**
      * GdlDockObject:long-name:
      *
      * A long descriptive name.
      */
-    g_object_class_install_property (
-        object_class, PROP_LONG_NAME,
+    properties[PROP_LONG_NAME] =
         g_param_spec_string ("long-name", _("Long name"),
                              _("Human readable name for the dock object"),
                              NULL,
-                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+    g_object_class_install_property (
+        object_class, PROP_LONG_NAME, properties[PROP_LONG_NAME]);
 
     /**
      * GdlDockObject:stock-id:
      *
      * A stock id to use for the icon of the dock object.
      */
-    g_object_class_install_property (
-        object_class, PROP_STOCK_ID,
+    properties[PROP_STOCK_ID] =
         g_param_spec_string ("stock-id", _("Stock Icon"),
                              _("Stock icon for the dock object"),
                              NULL,
-                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+    g_object_class_install_property (
+        object_class, PROP_STOCK_ID, properties[PROP_STOCK_ID]);
 
     /**
      * GdlDockObject:pixbuf-icon:
@@ -237,23 +246,27 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
      *
      * Since: 3.3.2
      */
-    g_object_class_install_property (
-        object_class, PROP_PIXBUF_ICON,
+    properties[PROP_PIXBUF_ICON] =
         g_param_spec_pointer ("pixbuf-icon", _("Pixbuf Icon"),
                               _("Pixbuf icon for the dock object"),
-                              G_PARAM_READWRITE));
+                              G_PARAM_READWRITE);
+
+    g_object_class_install_property (
+        object_class, PROP_PIXBUF_ICON, properties[PROP_PIXBUF_ICON]);
 
     /**
      * GdlDockObject:master:
      *
      * The master which manages all the objects in a dock ring
      */
-    g_object_class_install_property (
-        object_class, PROP_MASTER,
+    properties[PROP_MASTER] =
         g_param_spec_object ("master", _("Dock master"),
                              _("Dock master this dock object is bound to"),
                              GDL_TYPE_DOCK_MASTER,
-                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+    g_object_class_install_property (
+        object_class, PROP_MASTER, properties[PROP_MASTER]);
 
     widget_class->destroy = gdl_dock_object_destroy;
 
@@ -291,6 +304,7 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
 
     /**
      * GdlDockObject::dock:
+     * @item: The docked dock object.
      * @requestor: The widget to dock
      * @position: The position for the child
      * @other_data: (allow-none): Optional data giving additional information
@@ -451,6 +465,30 @@ gdl_dock_object_foreach_is_visible (GdlDockObject *object,
     if (!*visible && gtk_widget_get_visible (GTK_WIDGET (object))) *visible = TRUE;
 }
 
+static void
+gdl_dock_object_update_visibility (GdlDockObject *object)
+{
+    g_return_if_fail (object != NULL);
+
+    if (object && gdl_dock_object_is_automatic (object))
+    {
+        gboolean visible = FALSE;
+
+        gtk_container_foreach (GTK_CONTAINER (object),
+                               (GtkCallback) gdl_dock_object_foreach_is_visible,
+                               &visible);
+        object->priv->attached = visible;
+#ifndef GDL_DISABLE_DEPRECATED
+        if (visible)
+            object->deprecated_flags |= GDL_DOCK_ATTACHED;
+        else
+            object->deprecated_flags &= ~GDL_DOCK_ATTACHED;
+#endif
+        gtk_widget_set_visible (GTK_WIDGET (object), visible);
+    }
+    gdl_dock_object_layout_changed_notify (object);
+}
+
 static void
 gdl_dock_object_update_parent_visibility (GdlDockObject *object)
 {
@@ -459,23 +497,7 @@ gdl_dock_object_update_parent_visibility (GdlDockObject *object)
     g_return_if_fail (object != NULL);
 
     parent = gdl_dock_object_get_parent_object (object);
-    if (parent && gdl_dock_object_is_automatic (parent))
-    {
-        gboolean visible = FALSE;
-
-        gtk_container_foreach (GTK_CONTAINER (parent),
-                               (GtkCallback) gdl_dock_object_foreach_is_visible,
-                               &visible);
-        parent->priv->attached = visible;
-#ifndef GDL_DISABLE_DEPRECATED
-        if (visible)
-            parent->deprecated_flags |= GDL_DOCK_ATTACHED;
-        else
-            parent->deprecated_flags &= ~GDL_DOCK_ATTACHED;
-#endif
-        gtk_widget_set_visible (GTK_WIDGET (parent), visible);
-    }
-    gdl_dock_object_layout_changed_notify (object);
+    if (parent != NULL) gdl_dock_object_update_visibility (parent);
 }
 
 static void
@@ -853,6 +875,8 @@ gdl_dock_object_dock (GdlDockObject    *object,
 
     /* detach the requestor before docking */
     g_object_ref (requestor);
+    parent = gdl_dock_object_get_parent_object (requestor);
+    if (parent) g_object_ref (parent);
     gdl_dock_object_detach (requestor, FALSE);
 
     if (position != GDL_DOCK_NONE)
@@ -869,6 +893,11 @@ gdl_dock_object_dock (GdlDockObject    *object,
 #endif
     }
     /* Update visibility of automatic parents */
+    if (parent != NULL)
+    {
+        gdl_dock_object_update_visibility (parent);
+        g_object_unref (parent);
+    }
     gdl_dock_object_update_parent_visibility (GDL_DOCK_OBJECT (requestor));
 }
 
@@ -1183,6 +1212,8 @@ gdl_dock_object_set_name (GdlDockObject *object,
 
     g_free (object->priv->name);
     object->priv->name = g_strdup (name);
+
+    g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_NAME]);
 }
 
 /**
@@ -1222,6 +1253,8 @@ gdl_dock_object_set_long_name (GdlDockObject *object,
 
     g_free (object->priv->long_name);
     object->priv->long_name = g_strdup (name);
+
+    g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_LONG_NAME]);
 }
 
 /**
@@ -1259,6 +1292,8 @@ gdl_dock_object_set_stock_id (GdlDockObject *object,
 
     g_free (object->priv->stock_id);
     object->priv->stock_id = g_strdup (stock_id);
+
+    g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_STOCK_ID]);
 }
 
 /**
@@ -1296,6 +1331,8 @@ gdl_dock_object_set_pixbuf (GdlDockObject *object,
     g_return_if_fail (icon == NULL || GDK_IS_PIXBUF (icon));
 
     object->priv->pixbuf_icon =icon;
+
+    g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_PIXBUF_ICON]);
 }
 
 
