@@ -15,9 +15,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * Copied and adapted from ESidebar.[ch] from evolution
@@ -871,8 +869,7 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
     static const gchar button_style[] =
        "* {\n"
-           "-GtkWidget-focus-line-width : 1;\n"
-           "-GtkWidget-focus-padding : 1;\n"
+           "outline-width : 1px;\n"
            "padding: 0;\n"
        "}";
 
